#ifndef _CAVIMAGEDESCRIPTOR
#define _CAVIMAGEDESCRIPTOR

/*
 *
 * This is part of the ANL/MCS/FL CAVEav distribution.
 * written by Ivan R. Judson & Bob Olson
 *
 */

/** This is the ImageDescriptor class.  This class is designed to hold
  an image or a single frame of video.  
  */

#define MAXWID  640
#define MAXHEI  480
#define MAXDEP  4

class CAVImageDescriptor
{
public:
  /**@name Methods */
  //@{
    
  /// This accessor returns the width of the ImageDescriptor

  int GetWidth(void) { 
    return _width; 
  }

  /// This accessor returns the height of the ImageDescriptor
  
  int GetHeight(void) { 
    return _height; 
  }

  /// This accessor returns the depth of the ImageDescriptor

  int GetDepth(void) { 
    return _depth; 
  }

  /// This accessor sets the timestamp
	
  void SetTimestamp(unsigned long tstamp) { 
    _timestamp = tstamp; 
  }

  /// This accessor returns the address of the _data segment

  volatile unsigned char *GetData(void) { 
    return _data; 
  }

  /// This accessor lets you set the current size of the frames

  void SetSize(int width, int height, int depth) {
    _width = width; _height = height; _depth = depth;
  }

  //@}

protected:

  /**@name Data */
  //@{

  /// Width of the image

  volatile int _width;
    
  /// Height of the image

  volatile int _height;

  /// Depth of the image

  volatile int _depth;

  /// Size of the image

  volatile int _size;

  /// Timestamp of the last time this image was touched

  volatile unsigned long _timestamp;
    
  /// Block of memory to store this image

  volatile unsigned char _data[MAXWID * MAXHEI * MAXDEP];
  //@}
};

#endif
