#ifndef _FLSHAREDMEMORY
#define _FLSHAREDMEMORY

/*
 *
 * This is part of the ANL/MCS/FL software infrastructure.
 * written by Ivan R. Judson
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include <stdlib.h>

class FLSharedMemory 
{

public:
  FLSharedMemory(key_t key, int size, 
		  int mode = 0666) 
    {
      _key = key;
      _size = size;
      _mode = mode;

      shm_get_id();
      
      shm_attach();
    }

  FLSharedMemory(int id = 0)
    {
      _id = id;

      if(!_id)
	{
	  shm_get_id();
	}
      shm_attach();
    }

  ~FLSharedMemory(void)
    {
      if(shmctl(_id,IPC_RMID) < 0)
	{
	  perror("sharedMemory::DetachMemory: Could not remove shm");
	}
    }

  void *GetAddress(void) {
    return _buffer;
  }

  int GetID(void) {
    return _id;
  }

private:

  void shm_get_id(void) {
    if((_id = shmget(_key, _size, _mode | IPC_CREAT)) < 0) 
      perror("sharedMemory::MakeMemory: Unable to shmget memory.");
#ifdef DEBUG
    fprintf(stderr, "Got <ID %d> for <KEY %d> <SIZE %d> <MODE %d>\n",
	    _id, _key, _size, _mode);
#endif
  }
  
  void shm_attach(void) {
    if((long) (_buffer = shmat(_id, (char *)0, 0)) < 0) 
      perror("sharedMemory::MakeMemory: Unable to shmat memory.");
#ifdef DEBUG
    fprintf(stderr, "Got <ADDR %x>\n", _buffer);
#endif
  } 

  key_t _key;
  
  int _size;
  
  int _mode;
  
  int _id;
  
  void *_buffer;

};

#endif

