/*
 *
 * This is part of the ANL/MCS/FL CAVEav distribution.
 * written by Ivan R. Judson
 *
 */

#include <fileCAVcam.h>

fileCAVcam::fileCAVcam(char *fn, int width, int height, int depth,
			 float ex, float ey, float ez, 
			 float cx, float cy, float cz, 
			 float ux, float uy, float uz) : CAVcam(ex, ey, ez,
								 cx, cy, cz,
								 ux, uy, uz)
{
  fnbase = fn;
  _fb_width = width; _fb_height = height; _fb_depth = depth;
  _fb_size = _fb_width * _fb_height;
  _imagepool = new CAVImagePool(_fb_width, _fb_height, _fb_depth, 10);
  //  _pid = fork();
  
  //  if(_pid == 0)
  //    {
  //      while(1)
  //	PutPix();
  //    }
}

fileCAVcam::~fileCAVcam()
{
  //  kill(SIGKILL, _pid);
  delete _imagepool;
}

void fileCAVcam::Process()
{
  GLint buff_id[1];
  CAVImageDescriptor *image;
  int width, height;
  int nextIndex;
  GLint vpp[4];
  int x_adj, y_adj;

  if( _imagepool->_clientActive &&
      CAVEEye == CAVE_LEFT_EYE && CAVEMasterDisplay() &&
      !(_fc % _modFrames) && _captureFrames)
    {
      // Get the right place in the ring buffers to put the newest image
      nextIndex = (_imagepool->_nextImage + 1) % _imagepool->_nImages;
      _ImageToWrite = nextIndex;
      image = &(_imagepool->_images[nextIndex]);

      // Get these once
      width = image->GetWidth();
      height = image->GetHeight();

      // Get window info
      glGetIntegerv(GL_VIEWPORT, vpp);
      if(vpp[2] > width)
	x_adj = (vpp[2] - width) / 2;
      else 
	x_adj = 0;
      if(vpp[3] > height)
	y_adj = (vpp[3] - height) / 2;
      else 
	y_adj = 0;

#ifdef DEBUG
      fprintf(stderr, "Using buffer %d\n", nextIndex);
      fprintf(stderr, "Width < %d > Height < %d >\n", width, height);
      fprintf(stderr, "Shifting < %d %d > for viewport adjustment\n", 
	      x_adj, y_adj);
      fprintf(stderr, "Camera Position: < %f %f %f >\n", _eyex, _eyey, _eyez);
      fprintf(stderr, "Camera Forward: < %f %f %f >\n", _eyex + _centerx, 
	      _eyey + _centery, _eyez + _centerz);
      fprintf(stderr, "Camera Up: < %f %f %f >\n", _upx, _upy, _upz);
#endif
      glPushMatrix();
      {
	glMatrixMode(GL_MODELVIEW_MATRIX);
	glLoadIdentity();

      // Set Camera
      gluLookAt(_eyex, _eyey, _eyez, 
		_eyex + _centerx, _eyey + _centery, _eyez + _centerz, 
		_upx, _upy, _upz);

      glGetIntegerv(GL_DRAW_BUFFER, buff_id);
      glReadBuffer(buff_id[0]);

      // Draw the scene
      CallDraw();

      // Wait for all graphics to get drawn
      glFinish();

      // Read Pixels
      glReadPixels(0 + x_adj, 0 + y_adj, width, height, 
		   GL_RGB, GL_UNSIGNED_BYTE, 
		   (GLvoid *)image->GetData());
      }
      glPopMatrix();

      // Update ring buffers
      _imagepool->_nextImage = nextIndex;
      _imagepool->_currentSerial++;
    
      // Put them somewhere
      PutPix();
    }
}

void fileCAVcam::PutPix()
{
  FILE *fp;
  char fn[128];
  CAVImageDescriptor *image;
  int w, h, d;

  sprintf(fn, "%s%d.raw", fnbase, _fc++);

#ifdef DEBUG
  fprintf(stderr, "Opening %s to write to...", fn);
#endif

  image = &(_imagepool->_images[_ImageToWrite]);
  w = image->GetWidth();
  h = image->GetHeight();
  d = image->GetDepth();

  if((fp = fopen(fn, "w")) != NULL) 
    {
      if(fwrite((void *)image->GetData(), w * h * d, 1, fp)) 
	{
	  fclose(fp);
#ifdef DEBUG
	  fprintf(stderr, "closed.\n");
#endif
	} else {
	  perror("Can't write frame");
	}
    } else {
      perror("Can't open file");
    }
}

